import 'package:flutter/cupertino.dart';

import 'device_query.dart';
import 'device_query_data.dart';

class DeviceValue<T> {
  const DeviceValue({
    T? web,
    T? phone,
    T? tablet,
    T? desktop,
    T? tv,
    T? watch,
    T? fallback,
  })  : _web = web,
        _tablet = tablet,
        _phone = phone,
        _tv = tv,
        _watch = watch,
        _desktop = desktop,
        _fallback = fallback;

  final T? _fallback;
  T get fallback => _fallback!;

  final T? _web;
  T get web => _web ?? _fallback!;

  final T? _tablet;
  T get tablet => _tablet ?? _fallback!;

  final T? _phone;
  T get phone => _phone ?? _fallback!;

  final T? _watch;
  T get watch => _watch ?? _fallback!;

  final T? _tv;
  T get tv => _tv ?? _fallback!;

  final T? _desktop;
  T get desktop => _desktop ?? _fallback!;

  T getValue(BuildContext context) => maybeGetValue(context)!;

  T? maybeGetValue(BuildContext context) {
    final device = DeviceQuery.of(context);

    switch (device.deviceType) {
      case DeviceType.desktop:
        return _desktop;

      case DeviceType.phone:
        return _phone;

      case DeviceType.tablet:
        return _tablet;

      case DeviceType.tv:
        return _tv;

      case DeviceType.watch:
        return _watch;

      case DeviceType.web:
        return _web;

      default:
        return _fallback;
    }
  }
}
